
#ifndef KCONTACTS_EXPORT_H
#define KCONTACTS_EXPORT_H

#ifdef KCONTACTS_STATIC_DEFINE
#  define KCONTACTS_EXPORT
#  define KCONTACTS_NO_EXPORT
#else
#  ifndef KCONTACTS_EXPORT
#    ifdef KF5Contacts_EXPORTS
        /* We are building this library */
#      define KCONTACTS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define KCONTACTS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef KCONTACTS_NO_EXPORT
#    define KCONTACTS_NO_EXPORT 
#  endif
#endif

#ifndef KCONTACTS_DEPRECATED
#  define KCONTACTS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KCONTACTS_DEPRECATED_EXPORT
#  define KCONTACTS_DEPRECATED_EXPORT KCONTACTS_EXPORT KCONTACTS_DEPRECATED
#endif

#ifndef KCONTACTS_DEPRECATED_NO_EXPORT
#  define KCONTACTS_DEPRECATED_NO_EXPORT KCONTACTS_NO_EXPORT KCONTACTS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KCONTACTS_NO_DEPRECATED
#    define KCONTACTS_NO_DEPRECATED
#  endif
#endif

#endif
