// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KContacts_VERSION_H
#define KContacts_VERSION_H

#define KContacts_VERSION_STRING "5.5.3"
#define KContacts_VERSION_MAJOR 5
#define KContacts_VERSION_MINOR 5
#define KContacts_VERSION_PATCH 3
#define KContacts_VERSION ((5<<16)|(5<<8)|(3))

#endif
